// This file tests that the VM's call command correctly saves the return
// address and four virtual registers on the stack, correctly sets
// the ARG and LCL segments, and jumps to the called function.
//
// The translator should NOT emit bootstrap code for this test.
//
// After each translation of Call.vm, manually add the following to the end
// of your Call.asm:
//
//     (Call.func)
//     @Call.func
//     0;JMP
//
// This code will serve as the simulated callee.
//
// After running the test, verify that the CPU Emulator is in the midst of
// executing this infinite loop, which will be at the very end of the code
// in the program ROM.  I.e., the PC should point to one of the last two
// instructions in the program ROM.
//
// Also, after running the test, check that RAM[258] contains the correct
// return address.  The last thing that call does is jump to the callee,
// using '0;JMP', or a similar unconditional HACK JMP instruction that you
// used to jump to the callee.  Locate this instruction in the ROM window of
// the CPU Emulator.  The value in RAM[258] should match the address of the
// instruction FOLLOWING the '0;JMP' instruction.  Be careful to identify
// the correct '0;JMP' instruction; there will be three of them at the end
// of the code in the program ROM ---
//
// 1) The one used to jump to the callee.  This is the one to use for this
//    check. 
//
// 2) The one for the SHOULDNOTREACH infinite loop in Call.vm.
//
// 3) The one that you manually added to the end of Call.asm to simulate
//    the callee.
//
// Finally, after your translator has passed the tests above, add a second
// call at the end of Call.vm:
//
//     call Call.func 2
//
// translate Call.vm file, open Call.asm, and verify that your translator
// is emitting and using unique Hack labels for the return addresses for
// each of the calls.

load Call.asm,
output-file Call.out,
compare-to Call.cmp,
output-list RAM[0]%D1.6.1 RAM[1]%D1.6.1 RAM[2]%D1.6.1
             RAM[256]%D1.6.1 RAM[257]%D1.6.1
             RAM[259]%D1.6.1 RAM[260]%D1.6.1 RAM[261]%D1.6.1
             RAM[262]%D1.6.1 RAM[263]%D1.6.1;

// Set SP and virtual registers.

set RAM[0] 258,    // SP
set RAM[1] 12,     // LCL
set RAM[2] 34,     // ARG
set RAM[3] 56,     // THIS
set RAM[4] 78,     // THAT

// Set the stack and locations above the stack where the callee's frame
// will be established.

set RAM[256] 42,    // Callee's ARG 0
set RAM[257] 24,    // Callee's ARG 1
set RAM[258] -1,    // Will contain return address
set RAM[259] -1,    // Will contain saved LCL
set RAM[260] -1,    // Saved ARG
set RAM[261] -1,    // Saved THIS
set RAM[262] -1,    // Saved THAT
set RAM[263] -1,    // Should not change.

repeat 100 {
  ticktock;
}

output;
